==> bfgcardplay-0.0.38-py3-none-any.whl: bfgcardplay/test/test_opening_lead_card.py <==



HOLDING_SUIT_CONTRACT = {
    # Based on Klinger's Basic Bridge p138-9
    '95': ('9', '001'),
    '953': ('5', '005'),
    '96532': ('3', '004'),

    'T9': ('T', '001'),
    'T6': ('T', '001'),
    'T93': ('T', '003'),
    'T63': ('3', '007'),
    'T632': ('2', '004'),
    'T983': ('T', '020'),
    'T973': ('T', '021'),
    'T963': ('3', '004'),

    'JT': ('J', '001'),
    'J5': ('J', '001'),
    'JT6': ('J', '003'),
    'J52': ('2', '007'),
    'J542': ('2', '004'),
    'J9876': ('7', '004'),
    'J7542': ('4', '004'),
    'JT94': ('J', '020'),
    'JT84': ('J', '021'),
    'JT74': ('4', '004'),

    'QJ': ('Q', '001'),
    'Q4': ('Q', '001'),
    'QJ4': ('Q', '003'),
    'QT9': ('T', '008'),
    'QT4': ('4', '010'),
    'Q64': ('4', '007'),
    'QJT2': ('Q', '020'),
    'QJ92': ('Q', '021'),
    'QJ82': ('2', '004'),
    'QT98': ('T', '020'),
    'QT83': ('3', '004'),
    'Q9876': ('7', '004'),
    'Q8652': ('5', '004'),

    'KQ': ('K', '001'),
    'K2': ('K', '001'),
    'KQ5': ('K', '003'),
    'KJT': ('J', '008'),
    'KJ5': ('5', '010'),
    'KT9': ('T', '008'),
    'KT5': ('5', '010'),
    'K75': ('5', '007'),
    'KQJ2': ('K', '020'),
    'KQT2': ('K', '021'),
    'KQ92': ('K', '023'),
    'KJT2': ('J', '022'),
    'KJ92': ('2', '004'),
    'KT98': ('T', '020'),
    'KT84': ('4', '004'),
    'KQJ63': ('K', '020'),
    'KQT63': ('K', '021'),
    'KQ763': ('K', '023'),
    'K9873': ('7', '004'),
    'K8643': ('4', '004'),

    'AK': ('K', '002'),
    'A6': ('A', '001'),
    'AKQ': ('A', '013'),
    'AKJ': ('A', '013'),
    'AK3': ('A', '013'),
    'A93': ('A', '013'),
    'AKQ3': ('A', '013'),
    'AKJ3': ('A', '013'),
    'AK63': ('A', '013'),
    'AQJ3': ('A', '013'),
    'AQ63': ('A', '013'),
    'AJT3': ('A', '013'),
    'AJ63': ('A', '013'),
    'AT98': ('A', '013'),
    'AT52': ('A', '013'),
    'A987': ('A', '013'),
    'A963': ('A', '013'),
    'AKJ42': ('A', '013'),
    'AK742': ('A', '013'),
    'AQJ42': ('A', '013'),
    'AQT92': ('A', '013'),
    'AQT42': ('A', '013'),
    'AQ642': ('A', '013'),
    'AJT53': ('A', '013'),
    'AJ853': ('A', '013'),
    'AT983': ('A', '013'),
    'AT853': ('A', '013'),
    '53': ('5', '001'),
    '953': ('5', '005'),
    '9653': ('6', '014'),

    # Klinger Guide to Better Card Play p4
    '64': ('6', '001'),
    '92': ('9', '001'),
    '87': ('8', '001'),
    '632': ('3', '005'),
    '987': ('8', '005'),
    '853': ('5', '005'),
    'T9': ('T', '001'),
    'T6': ('T', '001'),
    'T92': ('T', '003'),
    'T82': ('2', '007'),

    'JT9': ('J', '003'),
    'JT8': ('J', '003'),
    'JT2': ('J', '003'),
    'J98': ('8', '007'),
    'J94': ('4', '007'),
    'J75': ('5', '007'),
    'J32': ('2', '007'),
    'JT': ('J', '001'),
    'J9': ('J', '001'),
    'J3': ('J', '001'),

    'QJ': ('Q', '001'),
    'QT': ('Q', '001'),
    'Q6': ('Q', '001'),
    'QJT': ('Q', '003'),
    'QJ5': ('Q', '003'),
    'Q94': ('4', '007'),
    'Q98': ('8', '007'),
    'QT2': ('2', '010'),
    'QT9': ('T', '008'),
    'Q43': ('3', '007'),

    'KQ': ('K', '001'),
    'KJ': ('K', '001'),
    'K7': ('K', '001'),
    'KJ2': ('2', '010'),
    'KJT': ('J', '008'),
    'KJ8': ('8', '010'),
    'KT9': ('T', '008'),
    'KQ3': ('K', '003'),
    'K98': ('8', '007'),
    'K74': ('4', '007'),

    'AK': ('K', '002'),
    'AT': ('A', '001'),
    'A4': ('A', '001'),
    'A93': ('A', '013'),
    'A87': ('A', '013'),
    'AT2': ('A', '013'),
    'AT9': ('A', '013'),
    'AKQ': ('A', '013'),
    'AK4': ('A', '013'),
    'AJT': ('A', '013'),

    # Klinger Guide to Better Card Play p9
    '8643': ('6', '014'),
    '86432': ('3', '004'),
    '9876': ('8', '014'),
    '98765': ('6', '004'),
    '6532': ('5', '014'),
    'T752': ('2', '004'),
    'T985': ('T', '020'),
    'T975': ('T', '021'),
    'T965': ('5', '004'),
    'T65432': ('4', '004'),
    'J864': ('4', '004'),
    'J987': ('7', '004'),
    'JT52': ('2', '004'),
    'JT84': ('J', '021'),
    'JT93': ('J', '020'),
    'Q753': ('3', '004'),
    'QT84': ('4', '004'),
    'QT94': ('T', '022'),
    'QJ62': ('2', '004'),
    'QJ95': ('Q', '021'),
    'QJ85': ('5', '004'),
    'QJT3': ('Q', '020'),
    'QJ987': ('Q', '021'),
    'QJ7652': ('6', '004'),
    'Q9875': ('7', '004'),
    'K852': ('2', '004'),
    'K9873': ('7', '004'),
    'KT832': ('3', '004'),
    'KT932': ('T', '022'),
    'KJ752': ('5', '004'),
    'KJ943': ('4', '004'),
    'KJT43': ('J', '022'),
    'KQ653': ('K', '023'),
    'KQT32': ('K', '021'),
    'KQJ43': ('K', '020'),
    'A874': ('A', '013'),
    'AT75': ('A', '013'),
    'AT95': ('A', '013'),
    'AJ96': ('A', '013'),
    'AJT6': ('A', '013'),
    'AQT98': ('A', '013'),
    'AQJ65': ('A', '013'),
    'AQ742': ('A', '013'),
    'AK53': ('A', '013'),
    'AKJ73': ('A', '013'),
    'AK987': ('A', '013'),
    'AKQ5': ('A', '013'),
    'A98742': ('A', '013'),
    'AJ987': ('A', '013'),
    'AKT98': ('A', '013'),
    # 'xxx': ('x', '000'),



    # Klinger Guide to Better Card Play p24
    '9742': ('7', '014'),
    'J7': ('J', '001'),
    'K83': ('3', '007'),
    'A964': ('A', '013'),
    '9': ('9', '015'),
    'KJ752': ('5', '004'),
    'Q9843': ('4', '004'),
    'QJ72': ('2', '004'),
    '832': ('3', '005'),
    'KQ754': ('K', '023'),
    '2': ('2', '015'),
    '873': ('7', '005'),
    'K763': ('3', '004'),
    '7': ('7', '015'),
    'K872': ('2', '004'),
    'QJT2': ('Q', '020'),
    '86': ('8', '001'),
    '98642': ('4', '004'),
}

HOLDING_NT_CONTRACT ={
    # Based on Klinger's Basic Bridge p138-9
    '95': ('9', '001'),
    '953': ('5', '005'),
    # '96532': ('3', '004'),

    'T9': ('T', '001'),
    'T6': ('T', '001'),
    'T93': ('T', '003'),
    'T63': ('3', '007'),
    'T632': ('2', '004'),
    # 'T983': ('T', '003'),
    # 'T973': ('T', '003'),
    'T963': ('3', '004'),

    'JT': ('J', '001'),
    'J5': ('J', '001'),
    'JT6': ('J', '003'),
    'J52': ('2', '007'),
    'J542': ('2', '004'),
    'J9876': ('7', '004'),
    'J7542': ('4', '004'),
    # 'JT94': ('J', '003'),
    # 'JT84': ('J', '003'),
    'JT74': ('4', '004'),

    'QJ': ('Q', '001'),
    'Q4': ('Q', '001'),
    'QJ4': ('Q', '003'),
    'QT9': ('T', '008'),
    'QT4': ('4', '010'),
    'Q64': ('4', '007'),
    'QJT2': ('Q', '020'),
    # 'QJ92': ('Q', '003'),
    'QJ82': ('2', '004'),
    'QT98': ('T', '020'),
    'QT83': ('3', '004'),
    'Q9876': ('7', '004'),
    'Q8652': ('5', '004'),

    'KQ': ('K', '001'),
    'K2': ('K', '001'),
    'KQ5': ('K', '003'),
    'KJT': ('J', '008'),
    'KJ5': ('5', '010'),
    'KT9': ('T', '008'),
    'KT5': ('5', '010'),
    'K75': ('5', '007'),
    # 'KQJ2': ('K', '003'),
    # 'KQT2': ('K', '003'),
    'KJT2': ('J', '022'),
    'KJ92': ('2', '004'),
    'KT98': ('T', '020'),
    'KT84': ('4', '004'),
    # 'KQJ63': ('K', '003'),
    # 'KQT63': ('K', '003'),
    'K9873': ('7', '004'),
    'K8643': ('4', '004'),

    'AK': ('K', '002'),
    'A6': ('A', '001'),
    'AKQ': ('A', '013'),
    'AKJ': ('A', '013'),
    'AK3': ('A', '013'),
    # 'AKQ3': ('A', '013'),
    # 'AKJ3': ('A', '013'),
    # 'AKJ42': ('A', '013'),
    '53': ('5', '001'),
    '953': ('5', '005'),
    # '9653': ('6', '014'),

    'KQ92': ('2', '004'),
    'KQ763': ('6', '004'),

    'A93': ('3', '007'),
    'AK63': ('3', '004'),
    'AQJ3': ('Q', '022'),
    'AQ63': ('3', '004'),
    'AJT3': ('J', '022'),
    'AJ63': ('3', '004'),
    'AT98': ('T', '020'),
    'AT52': ('2', '004'),
    'A987': ('7', '004'),
    'A963': ('3', '004'),
    'AK742': ('4', '004'),
    'AQJ42': ('Q', '022'),
    'AQT92': ('T', '022'),
    'AQT42': ('4', '004'),
    'AQ642': ('4', '004'),
    'AJT53': ('J', '022'),
    'AJ853': ('5', '004'),
    'AT983': ('T', '020'),
    'AT853': ('5', '004'),

    # Klinger Guide to Better Card Play p4
    '64': ('6', '001'),
    '92': ('9', '001'),
    '87': ('8', '001'),
    '632': ('3', '005'),
    '987': ('8', '005'),
    '853': ('5', '005'),
    'T9': ('T', '001'),
    'T6': ('T', '001'),
    'T92': ('T', '003'),
    'T82': ('2', '007'),

    'JT9': ('J', '003'),
    'JT8': ('J', '003'),
    'JT2': ('J', '003'),
    'J98': ('8', '007'),
    'J94': ('4', '007'),
    'J75': ('5', '007'),
    'J32': ('2', '007'),
    'JT': ('J', '001'),
    'J9': ('J', '001'),
    'J3': ('J', '001'),

    'QJ': ('Q', '001'),
    'QT': ('Q', '001'),
    'Q6': ('Q', '001'),
    'QJT': ('Q', '003'),
    'QJ5': ('Q', '003'),
    'Q94': ('4', '007'),
    'Q98': ('8', '007'),
    'QT2': ('2', '010'),
    'QT9': ('T', '008'),
    'Q43': ('3', '007'),

    'KQ': ('K', '001'),
    'KJ': ('K', '001'),
    'K7': ('K', '001'),
    'KJ2': ('2', '010'),
    'KJT': ('J', '008'),
    'KJ8': ('8', '010'),
    'KT9': ('T', '008'),
    'KQ3': ('K', '003'),
    'K98': ('8', '007'),
    'K74': ('4', '007'),

    'AK': ('K', '002'),
    'AT': ('A', '001'),
    'A4': ('A', '001'),
    'A93': ('3', '007'),
    'A87': ('7', '007'),
    'AT2': ('2', '007'),
    'AT9': ('9', '007'),
    'AKQ': ('A', '013'),
    'AK4': ('A', '013'),
    'AJT': ('T', '007'),

    # Klinger Guide to Better Card Play p9
    '8643': ('6', '014'),
    '86432': ('3', '004'),
    '9876': ('8', '014'),
    '98765': ('6', '004'),
    '6532': ('5', '014'),
    'T752': ('2', '004'),
    'T985': ('T', '020'),
    'T975': ('T', '021'),
    'T965': ('5', '004'),
    'T65432': ('4', '004'),
    'J864': ('4', '004'),
    'J987': ('7', '004'),
    'JT52': ('2', '004'),
    'JT84': ('J', '021'),
    'JT93': ('J', '020'),
    'Q753': ('3', '004'),
    'QT84': ('4', '004'),
    'QT94': ('T', '022'),
    'QJ62': ('2', '004'),
    'QJ95': ('Q', '021'),
    'QJ85': ('5', '004'),
    'QJT3': ('Q', '020'),
    'QJ987': ('Q', '021'),
    'QJ7652': ('6', '004'),
    'Q9875': ('7', '004'),
    'K852': ('2', '004'),
    'K9873': ('7', '004'),
    'KT832': ('3', '004'),
    'KT932': ('T', '022'),
    'KJ752': ('5', '004'),
    'KJ943': ('4', '004'),
    'KJT43': ('J', '022'),
    'KQ653': ('5', '004'),
    'KQT32': ('K', '021'),
    'KQJ43': ('K', '020'),
    'A874': ('4', '004'),
    'AT75': ('5', '004'),
    'AT95': ('T', '022'),
    'AJ96': ('6', '004'),
    'AJT6': ('J', '022'),
    'AQT98': ('T', '020'),
    'AQJ65': ('Q', '022'),
    'AQ742': ('4', '004'),
    'AK53': ('3', '004'),
    'AK987': ('8', '004'),
    'AKJ73': ('A', '021'),
    'AKQ5': ('A', '020'),
    'A98742': ('7', '004'),
    'AJ987': ('8', '004'),
    'AKT98': ('T', '020'),

    # Klinger Guide to Better Card Play p15
    'AJT94': ('J', '020'),
    'QJ982': ('Q', '021'),
    'K9753': ('5', '004'),
    'QT982': ('T', '020'),
    'JT986': ('J', '020'),
    # 'xxx': ('x', 'xxx'),
    # 'xxx': ('x', 'xxx'),
    # 'xxx': ('x', 'xxx'),
    # 'xxx': ('x', 'xxx'),

}

def test_lead_suit_contract():
    """Test the lead card returned in a suit contract."""
    # Arbitrarily use spade suit and 'H' as the contract suit
    contract = Contract('4H')
    for holding, (lead_card, reason) in HOLDING_SUIT_CONTRACT.items():
        cards = [Card(card_name+'S') for card_name in holding]
        selected_card = opening_lead_card(cards, contract)
        assert selected_card == Card(lead_card+'S')
        assert selected_card.selection_reason == reason

def test_lead_nt_contract():
    """Test the lead card returned in a NT contract."""
    contract = Contract('3NT')
    for holding, (lead_card, reason) in HOLDING_NT_CONTRACT.items():
        cards = [Card(card_name+'S') for card_name in holding]
        selected_card = opening_lead_card(cards, contract)
        assert selected_card == Card(lead_card+'S')
        assert selected_card.selection_reason == reason

==> bfgcardplay-1.0.8-py3-none-any.whl: bfgcardplay/test/test_opening_lead_card.py <==
from bridgeobjects import Contract, Card
from ..source.opening_lead_card import opening_lead_card

SUIT_CONTRACT_TESTA = {
    '96532': ('3', '004'),
}

SUIT_CONTRACT_TEST = {
    # Based on Klinger's Basic Bridge p138-9
    '95': ('9', '001'),
    '953': ('5', '005'),
    '96532': ('3', '004'),

    'T9': ('T', '001'),
    'T6': ('T', '001'),
    'T93': ('T', '003'),
    'T63': ('3', '007'),
    'T632': ('2', '004'),
    'T983': ('T', '020'),
    'T973': ('T', '021'),
    'T963': ('3', '004'),

    'JT': ('J', '001'),
    'J5': ('J', '001'),
    'JT6': ('J', '003'),
    'J52': ('2', '007'),
    'J542': ('2', '004'),
    'J9876': ('7', '004'),
    'J7542': ('4', '004'),
    'JT94': ('J', '020'),
    'JT84': ('J', '021'),
    'JT74': ('4', '004'),

    'QJ': ('Q', '001'),
    'Q4': ('Q', '001'),
    'QJ4': ('Q', '003'),
    'QT9': ('T', '008'),
    'QT4': ('4', '010'),
    'Q64': ('4', '007'),
    'QJT2': ('Q', '020'),
    'QJ92': ('Q', '021'),
    'QJ82': ('2', '004'),
    'QT98': ('T', '020'),
    'QT83': ('3', '004'),
    'Q9876': ('7', '004'),
    'Q8652': ('5', '004'),

    'KQ': ('K', '001'),
    'K2': ('K', '001'),
    'KQ5': ('K', '003'),
    'KJT': ('J', '008'),
    'KJ5': ('5', '010'),
    'KT9': ('T', '008'),
    'KT5': ('5', '010'),
    'K75': ('5', '007'),
    'KQJ2': ('K', '020'),
    'KQT2': ('K', '021'),
    'KQ92': ('K', '023'),
    'KJT2': ('J', '022'),
    'KJ92': ('2', '004'),
    'KT98': ('T', '020'),
    'KT84': ('4', '004'),
    'KQJ63': ('K', '020'),
    'KQT63': ('K', '021'),
    'KQ763': ('K', '023'),
    'K9873': ('7', '004'),
    'K8643': ('4', '004'),

    'AK': ('K', '002'),
    'A6': ('A', '001'),
    'AKQ': ('A', '013'),
    'AKJ': ('A', '013'),
    'AK3': ('A', '013'),
    'A93': ('A', '013'),
    'AKQ3': ('A', '013'),
    'AKJ3': ('A', '013'),
    'AK63': ('A', '013'),
    'AQJ3': ('A', '013'),
    'AQ63': ('A', '013'),
    'AJT3': ('A', '013'),
    'AJ63': ('A', '013'),
    'AT98': ('A', '013'),
    'AT52': ('A', '013'),
    'A987': ('A', '013'),
    'A963': ('A', '013'),
    'AKJ42': ('A', '013'),
    'AK742': ('A', '013'),
    'AQJ42': ('A', '013'),
    'AQT92': ('A', '013'),
    'AQT42': ('A', '013'),
    'AQ642': ('A', '013'),
    'AJT53': ('A', '013'),
    'AJ853': ('A', '013'),
    'AT983': ('A', '013'),
    'AT853': ('A', '013'),
    '53': ('5', '001'),
    '953': ('5', '005'),
    '9653': ('6', '014'),

    # Klinger Guide to Better Card Play p4
    '64': ('6', '001'),
    '92': ('9', '001'),
    '87': ('8', '001'),
    '632': ('3', '005'),
    '987': ('8', '005'),
    '853': ('5', '005'),
    'T9': ('T', '001'),
    'T6': ('T', '001'),
    'T92': ('T', '003'),
    'T82': ('2', '007'),

    'JT9': ('J', '003'),
    'JT8': ('J', '003'),
    'JT2': ('J', '003'),
    'J98': ('8', '007'),
    'J94': ('4', '007'),
    'J75': ('5', '007'),
    'J32': ('2', '007'),
    'JT': ('J', '001'),
    'J9': ('J', '001'),
    'J3': ('J', '001'),

    'QJ': ('Q', '001'),
    'QT': ('Q', '001'),
    'Q6': ('Q', '001'),
    'QJT': ('Q', '003'),
    'QJ5': ('Q', '003'),
    'Q94': ('4', '007'),
    'Q98': ('8', '007'),
    'QT2': ('2', '010'),
    'QT9': ('T', '008'),
    'Q43': ('3', '007'),

    'KQ': ('K', '001'),
    'KJ': ('K', '001'),
    'K7': ('K', '001'),
    'KJ2': ('2', '010'),
    'KJT': ('J', '008'),
    'KJ8': ('8', '010'),
    'KT9': ('T', '008'),
    'KQ3': ('K', '003'),
    'K98': ('8', '007'),
    'K74': ('4', '007'),

    'AK': ('K', '002'),
    'AT': ('A', '001'),
    'A4': ('A', '001'),
    'A93': ('A', '013'),
    'A87': ('A', '013'),
    'AT2': ('A', '013'),
    'AT9': ('A', '013'),
    'AKQ': ('A', '013'),
    'AK4': ('A', '013'),
    'AJT': ('A', '013'),

    # Klinger Guide to Better Card Play p9
    '8643': ('6', '014'),
    '86432': ('3', '004'),
    '9876': ('8', '014'),
    '98765': ('6', '004'),
    '6532': ('5', '014'),
    'T752': ('2', '004'),
    'T985': ('T', '020'),
    'T975': ('T', '021'),
    'T965': ('5', '004'),
    'T65432': ('4', '004'),
    'J864': ('4', '004'),
    'J987': ('7', '004'),
    'JT52': ('2', '004'),
    'JT84': ('J', '021'),
    'JT93': ('J', '020'),
    'Q753': ('3', '004'),
    'QT84': ('4', '004'),
    'QT94': ('T', '022'),
    'QJ62': ('2', '004'),
    'QJ95': ('Q', '021'),
    'QJ85': ('5', '004'),
    'QJT3': ('Q', '020'),
    'QJ987': ('Q', '021'),
    'QJ7652': ('6', '004'),
    'Q9875': ('7', '004'),
    'K852': ('2', '004'),
    'K9873': ('7', '004'),
    'KT832': ('3', '004'),
    'KT932': ('T', '022'),
    'KJ752': ('5', '004'),
    'KJ943': ('4', '004'),
    'KJT43': ('J', '022'),
    'KQ653': ('K', '023'),
    'KQT32': ('K', '021'),
    'KQJ43': ('K', '020'),
    'A874': ('A', '013'),
    'AT75': ('A', '013'),
    'AT95': ('A', '013'),
    'AJ96': ('A', '013'),
    'AJT6': ('A', '013'),
    'AQT98': ('A', '013'),
    'AQJ65': ('A', '013'),
    'AQ742': ('A', '013'),
    'AK53': ('A', '013'),
    'AKJ73': ('A', '013'),
    'AK987': ('A', '013'),
    'AKQ5': ('A', '013'),
    'A98742': ('A', '013'),
    'AJ987': ('A', '013'),
    'AKT98': ('A', '013'),
    # 'xxx': ('x', '000'),



    # Klinger Guide to Better Card Play p24
    '9742': ('7', '014'),
    'J7': ('J', '001'),
    'K83': ('3', '007'),
    'A964': ('A', '013'),
    '9': ('9', '015'),
    'KJ752': ('5', '004'),
    'Q9843': ('4', '004'),
    'QJ72': ('2', '004'),
    '832': ('3', '005'),
    'KQ754': ('K', '023'),
    '2': ('2', '015'),
    '873': ('7', '005'),
    'K763': ('3', '004'),
    '7': ('7', '015'),
    'K872': ('2', '004'),
    'QJT2': ('Q', '020'),
    '86': ('8', '001'),
    '98642': ('4', '004'),
}


NT_CONTRACT_TESTA ={
    'AT95': ('T', '022'),
}

NT_CONTRACT_TEST ={
    # Based on Klinger's Basic Bridge p138-9
    '95': ('9', '001'),
    '953': ('5', '005'),
    '96532': ('3', '004'),

    'T9': ('T', '001'),
    'T6': ('T', '001'),
    'T93': ('T', '003'),
    'T63': ('3', '007'),
    'T632': ('2', '004'),
    # 'T983': ('T', '003'),
    # 'T973': ('T', '003'),
    'T963': ('3', '004'),

    'JT': ('J', '001'),
    'J5': ('J', '001'),
    'JT6': ('J', '003'),
    'J52': ('2', '007'),
    'J542': ('2', '004'),
    'J9876': ('7', '004'),
    'J7542': ('4', '004'),
    # 'JT94': ('J', '003'),
    # 'JT84': ('J', '003'),
    'JT74': ('4', '004'),

    'QJ': ('Q', '001'),
    'Q4': ('Q', '001'),
    'QJ4': ('Q', '003'),
    'QT9': ('T', '008'),
    'QT4': ('4', '010'),
    'Q64': ('4', '007'),
    'QJT2': ('Q', '020'),
    # 'QJ92': ('Q', '003'),
    'QJ82': ('2', '004'),
    'QT98': ('T', '020'),
    'QT83': ('3', '004'),
    'Q9876': ('7', '004'),
    'Q8652': ('5', '004'),

    'KQ': ('K', '001'),
    'K2': ('K', '001'),
    'KQ5': ('K', '003'),
    'KJT': ('J', '008'),
    'KJ5': ('5', '010'),
    'KT9': ('T', '008'),
    'KT5': ('5', '010'),
    'K75': ('5', '007'),
    # 'KQJ2': ('K', '003'),
    # 'KQT2': ('K', '003'),
    'KJT2': ('J', '022'),
    'KJ92': ('2', '004'),
    'KT98': ('T', '020'),
    'KT84': ('4', '004'),
    # 'KQJ63': ('K', '003'),
    # 'KQT63': ('K', '003'),
    'K9873': ('7', '004'),
    'K8643': ('4', '004'),

    'AK': ('K', '002'),
    'A6': ('A', '001'),
    'AKQ': ('A', '013'),
    'AKJ': ('A', '013'),
    'AK3': ('A', '013'),
    # 'AKQ3': ('A', '013'),
    # 'AKJ3': ('A', '013'),
    # 'AKJ42': ('A', '013'),
    '53': ('5', '001'),
    '953': ('5', '005'),
    # '9653': ('6', '014'),

    'KQ92': ('2', '004'),
    'KQ763': ('6', '004'),

    'A93': ('3', '007'),
    'AK63': ('3', '004'),
    'AQJ3': ('Q', '022'),
    'AQ63': ('3', '004'),
    'AJT3': ('J', '022'),
    'AJ63': ('3', '004'),
    'AT98': ('T', '020'),
    'AT52': ('2', '004'),
    'A987': ('7', '004'),
    'A963': ('3', '004'),
    'AK742': ('4', '004'),
    'AQJ42': ('Q', '022'),
    'AQT92': ('T', '022'),
    'AQT42': ('4', '004'),
    'AQ642': ('4', '004'),
    'AJT53': ('J', '022'),
    'AJ853': ('5', '004'),
    'AT983': ('T', '020'),
    'AT853': ('5', '004'),

    # Klinger Guide to Better Card Play p4
    '64': ('6', '001'),
    '92': ('9', '001'),
    '87': ('8', '001'),
    '632': ('3', '005'),
    '987': ('8', '005'),
    '853': ('5', '005'),
    'T9': ('T', '001'),
    'T6': ('T', '001'),
    'T92': ('T', '003'),
    'T82': ('2', '007'),

    'JT9': ('J', '003'),
    'JT8': ('J', '003'),
    'JT2': ('J', '003'),
    'J98': ('8', '007'),
    'J94': ('4', '007'),
    'J75': ('5', '007'),
    'J32': ('2', '007'),
    'JT': ('J', '001'),
    'J9': ('J', '001'),
    'J3': ('J', '001'),

    'QJ': ('Q', '001'),
    'QT': ('Q', '001'),
    'Q6': ('Q', '001'),
    'QJT': ('Q', '003'),
    'QJ5': ('Q', '003'),
    'Q94': ('4', '007'),
    'Q98': ('8', '007'),
    'QT2': ('2', '010'),
    'QT9': ('T', '008'),
    'Q43': ('3', '007'),

    'KQ': ('K', '001'),
    'KJ': ('K', '001'),
    'K7': ('K', '001'),
    'KJ2': ('2', '010'),
    'KJT': ('J', '008'),
    'KJ8': ('8', '010'),
    'KT9': ('T', '008'),
    'KQ3': ('K', '003'),
    'K98': ('8', '007'),
    'K74': ('4', '007'),

    'AK': ('K', '002'),
    'AT': ('A', '001'),
    'A4': ('A', '001'),
    'A93': ('3', '007'),
    'A87': ('7', '007'),
    'AT2': ('2', '007'),
    'AT9': ('9', '007'),
    'AKQ': ('A', '013'),
    'AK4': ('A', '013'),
    'AJT': ('T', '007'),

    # Klinger Guide to Better Card Play p9
    '8643': ('6', '014'),
    '86432': ('3', '004'),
    '9876': ('8', '014'),
    '98765': ('6', '004'),
    '6532': ('5', '014'),
    'T752': ('2', '004'),
    'T985': ('T', '020'),
    'T975': ('T', '021'),
    'T965': ('5', '004'),
    'T65432': ('4', '004'),
    'J864': ('4', '004'),
    'J987': ('7', '004'),
    'JT52': ('2', '004'),
    'JT84': ('J', '021'),
    'JT93': ('J', '020'),
    'Q753': ('3', '004'),
    'QT84': ('4', '004'),
    'QT94': ('T', '022'),
    'QJ62': ('2', '004'),
    'QJ95': ('Q', '021'),
    'QJ85': ('5', '004'),
    'QJT3': ('Q', '020'),
    'QJ987': ('Q', '021'),
    'QJ7652': ('6', '004'),
    'Q9875': ('7', '004'),
    'K852': ('2', '004'),
    'K9873': ('7', '004'),
    'KT832': ('3', '004'),
    'KT932': ('T', '022'),
    'KJ752': ('5', '004'),
    'KJ943': ('4', '004'),
    'KJT43': ('J', '022'),
    'KQ653': ('5', '004'),
    'KQT32': ('K', '021'),
    'KQJ43': ('K', '020'),
    'A874': ('4', '004'),
    'AT75': ('5', '004'),
    'AT95': ('T', '022'),
    'AJ96': ('6', '004'),
    'AJT6': ('J', '022'),
    'AQT98': ('T', '020'),
    'AQJ65': ('Q', '022'),
    'AQ742': ('4', '004'),
    'AK53': ('3', '004'),
    'AK987': ('8', '004'),
    'AKJ73': ('A', '021'),
    'AKQ5': ('A', '020'),
    'A98742': ('7', '004'),
    'AJ987': ('8', '004'),
    'AKT98': ('T', '020'),

    # Klinger Guide to Better Card Play p15
    'AJT94': ('J', '020'),
    'QJ982': ('Q', '021'),
    'K9753': ('5', '004'),
    'QT982': ('T', '020'),
    'JT986': ('J', '020'),
    # 'xxx': ('x', 'xxx'),
    # 'xxx': ('x', 'xxx'),
    # 'xxx': ('x', 'xxx'),
    # 'xxx': ('x', 'xxx'),

}

def test_lead_suit_contract():
    """Test the lead card returned in a suit contract."""
    # Arbitrarily use spade suit and 'H' as the contract suit
    contract = Contract('4H')
    for holding, (lead_card, reason) in SUIT_CONTRACT_TEST.items():
        cards = [Card(card_name+'S') for card_name in holding]
        selected_card = opening_lead_card(cards, contract)
        assert selected_card == Card(lead_card+'S'), cards
        assert selected_card.selection_reason == reason, cards

def test_lead_nt_contract():
    """Test the lead card returned in a NT contract."""
    contract = Contract('3NT')
    for holding, (lead_card, reason) in NT_CONTRACT_TEST.items():
        cards = [Card(card_name+'S') for card_name in holding]
        selected_card = opening_lead_card(cards, contract)
        assert selected_card == Card(lead_card+'S'), cards
        assert selected_card.selection_reason == reason, cards
